﻿Public Class GeometricMedian
    Implements Strategy

    ' Lowest Limit till which we are going 
    ' to run the main while loop 
    ' Lower the Limit higher the accuracy 
    Dim lower_limit As Double = 0.01

    Public Function computeAverage(numbers() As Object) As Object Implements Strategy.computeAverage
        Dim numbersAsPoints() As Point = numbers.Select(Function(o) New Point(CInt(o.ToString.Split(","c)(0)), CInt(o.ToString.Split(","c)(1)))).ToArray
		Return geometricMedian(Array.ConvertAll(numbersAsPoints, Function(p) New PointF(p.X, p.Y)), numbersAsPoints.Count)
	End Function

	' Function to return the sum of Euclidean 
	' Distances 
	Private Function distSum(ByVal p As PointF, ByVal arr() As PointF, ByVal n As Integer) As Double
		Dim sum As Double = 0
		For i As Integer = 0 To n - 1
			Dim distx As Double = Math.Abs(arr(i).X - p.X)
			Dim disty As Double = Math.Abs(arr(i).Y - p.Y)
			sum += Math.Sqrt((distx * distx) + (disty * disty))
		Next i

		' Return the sum of Euclidean Distances 
		Return sum
	End Function

	' Function to calculate the required 
	' geometric median 
	Private Function geometricMedian(ByVal arr() As PointF, ByVal n As Integer) As Object

		' Current x coordinate and y coordinate 
		Dim current_point As PointF

		For i As Integer = 0 To n - 1
			current_point.X += arr(i).X
			current_point.Y += arr(i).Y
		Next i

		' Here current_point becomes the 
		' Geographic MidPoint 
		' Or Center of Gravity of equal 
		' discrete mass distributions 
		current_point.X /= n
		current_point.Y /= n

		' minimum_distance becomes sum of 
		' all distances from MidPoint to 
		' all given points 
		Dim minimum_distance As Double = distSum(current_point, arr, n)

		Dim k As Integer = 0
		Do While k < n
			Dim i As Integer = 0
			Do While i < n And i <> k
				Dim newpoint As PointF
				newpoint.X = arr(i).X
				newpoint.Y = arr(i).Y
				Dim newd As Double = distSum(newpoint, arr, n)
				If newd < minimum_distance Then
					minimum_distance = newd
					current_point.X = newpoint.X
					current_point.Y = newpoint.Y
				End If
				i += 1
			Loop
			k += 1
		Loop

		' Assume test_distance to be 1000 
		Dim test_distance As Double = 1000
		Dim flag As Integer = 0

		' Test loop for approximation starts here 
		Do While test_distance > lower_limit

			flag = 0

			' Loop for iterating over all 4 neighbours 
			For i As Integer = 0 To 3

				Dim test_point() As PointF = {current_point, current_point, current_point, current_point}
				test_point(0).X -= CSng(test_distance)
				test_point(1).Y -= CSng(test_distance)
				test_point(2).X += CSng(test_distance)
				test_point(3).Y += CSng(test_distance)

				' Finding Neighbours done 
				Dim newpoint As PointF
				newpoint.X = CSng(current_point.X + CDbl(test_distance) * test_point(i).X)
				newpoint.Y = CSng(current_point.Y + CDbl(test_distance) * test_point(i).Y)

				' New sum of Euclidean distances 
				' from the neighbor to the given 
				' data points 
				Dim newd As Double = distSum(newpoint, arr, n)

				If newd < minimum_distance Then

					' Approximating and changing 
					' current_point 
					minimum_distance = newd
					current_point.X = newpoint.X
					current_point.Y = newpoint.Y
					flag = 1
					Exit For
				End If
			Next i

			' This means none of the 4 neighbours 
			' has the new minimum distance, hence 
			' we divide by 2 and reiterate while 
			' loop for better approximation 
			If flag = 0 Then
				test_distance /= 2
			End If
		Loop

		'cout << "Geometric Median = ("
		'<< current_point.x << ", "
		'<< current_point.y << ")"; 
		'cout << " with minimum distance = "
		'<< minimum_distance; 
		'
		Dim min As String
		If minimum_distance Mod 1 = 0 Then
			min = minimum_distance.ToString
		Else
			min = minimum_distance.ToString("n5")
		End If

		Return "(" & CInt(current_point.X) & "," & CInt(current_point.Y) & ") with minimum distance = " & min
	End Function



	Public Overrides Function ToString() As String Implements Strategy.ToString
        Return "Geometric median"
    End Function

End Class
